/*
  ChatRep.cpp - Append affiliation to name in the Chat list (Private Channels).

  Jason Hood, 10 & 11 June, 2014.

  v1.01, 14 June, 2014:
  - fix for no server support or missing the last player (EDI in CharSel_Hook
    contains remaining length, not overall length);
  - fix for SP (EDI is 1).

  v1.02, 15 June, 2014:
  - fix the above fixes (smaller than 12, not smaller-equal).

  v1.03, 17 to 19 June, 2014:
  + remove the old ship id on entering a base, allowing affiliation to change;
  + change ship name to reflect new affiliation;
  + show changed affiliation if Chat window is open.

  v1.04, 22 June, 2014:
  - remove ship id on landing, not entering base.

  v1.05, 11 February, 2015:
  - fix sorting.
*/

#include "Common.h"
#include "patch.h"
#include <string>
#include <map>
#include <set>


#define ADDR_LIST      ((PDWORD)(0x465cfc+1))
#define ADDR_ARROW     ((PWORD)  0x466da2)
#define ADDR_DIR       ((PBYTE)  0x4681a2)
#define ADDR_SETNAME   ((PDWORD)(0x4ee784+1))
#define ADDR_TARGET    ((PBYTE)  0x4efe40)
#define ADDR_SETAFF    ((PBYTE)  0x53e323)
#define ADDR_CHARSEL   ((PDWORD) 0x5e66ac)
#define ADDR_LAND      ((PDWORD) 0x5e6760)
#define ADDR_SETREP    ((PDWORD) 0x5e677c)
#define ADDR_NEWPLAYER ((PDWORD) 0x5e682c)
#define ADDR_UPDAFF    ((PDWORD) 0x5e6844)
#define ADDR_SORT      ((PDWORD) 0x6115a8)


typedef std::map<UINT,int> PlayerRepMap;
typedef std::set<UINT>	   ShipSet;

DWORD List_Old, List_Ret, Sort_Old;
DWORD NewPlayer_Old, SetRep_Old, CharSel_Old;
DWORD SetName_Old, UpdAff_Old;
DWORD Land_Old;
PlayerRepMap PlayerRep;
UINT  player, target;
ShipSet ships;


typedef UINT (*TGetString)( LPVOID, UINT, LPWSTR, UINT );
TGetString GetString = (TGetString)0x4347e0;


void FASTCALL AppendRep( std::wstring& name, UINT player )
{
  int  rep;
  UINT fac;

  rep = PlayerRep[player];
  Reputation::Vibe::GetAffiliation( rep, fac );
  if (fac != -1)
  {
    WCHAR buf[64];
    wcscpy( buf, L" - " );
    GetString( NULL, Reputation::get_short_name( fac ), buf+3, 64-3 );
    name += buf;
  }
}


NAKED
void List_Hook()
{
  __asm {
	pop	List_Ret
	call	List_Old
	mov	edx,[esp+0xA8]
	mov	ecx,[edi+4]
	mov	edx,[edx]
	mov	ecx,[ecx+8]
	add	ecx,4
	call	AppendRep
	jmp	List_Ret
	align	16
  }
}


NAKED
void Dir_Hook()
{
  __asm {
	test	eax,eax
	jnz	done
	lea	edx,[ecx+1]
	and	dl,3
	ret
  done:
	test	cl,cl
	setz	dl
	ret
  }
}


int FASTCALL sort_affiliation( UINT p1, UINT p2 )
{
  int	r1, r2;
  UINT	f1, f2;
  WCHAR n1[64], n2[64];

  r1 = PlayerRep[p1];
  r2 = PlayerRep[p2];
  Reputation::Vibe::GetAffiliation( r1, f1 );
  Reputation::Vibe::GetAffiliation( r2, f2 );
  if (f1 == f2)
    return 0;
  if (f1 == -1)
    return -1;
  if (f2 == -1)
    return 1;

  GetString( NULL, Reputation::get_short_name( f1 ), n1, 64 );
  GetString( NULL, Reputation::get_short_name( f2 ), n2, 64 );
  return _wcsicmp( n1, n2 );
}


NAKED
void Sort_Hook()
{
  __asm {
	mov	eax,[esp+0x38]
	cmp	byte ptr [eax+0x3ac],2
	jb	done
	mov	ecx,[esp+4]
	mov	edx,[esp+8]
	pushfd
	mov	ecx,[ecx]
	mov	edx,[edx]
	call	sort_affiliation
	test	eax,eax
	jz	d
	popfd
	jne	r
	neg	eax
  r:	ret
  d:	popfd
  done:
	jmp	Sort_Old
	align	16
  }
}


NAKED
void NewPlayer_Hook()
{
  __asm {
	cmp	byte ptr [esp+16],0	// existing player?
	jne	done
	mov	eax,[esp+12]
	mov	player,eax
  done:
	jmp	NewPlayer_Old
	align	16
  }
}


void FASTCALL MapPlayerRep( int rep )
{
  PlayerRep[player] = rep;
  player = 0;
}


NAKED
void SetRep_Hook()
{
  __asm {
	mov	eax,ds:[0x66da7c]
	inc	dword ptr [eax+0x14]	// update other players' Chat affil.
	cmp	player,0
	je	done
	mov	ecx,[esp+8]
	mov	ecx,[ecx]
	call	MapPlayerRep
  done:
	jmp	SetRep_Old
	align	16
  }
}


NAKED
void CharSel_Hook()
{
  __asm {
	push	esi
	push	edi
	cmp	edi,12
	jl	done
  nxt:
	mov	eax,[esi]
	mov	ecx,[esi+4]
	mov	player,eax
	call	MapPlayerRep
	push	[esi+8]
	push	0
	push	[esi+4]
	push	esp
	push	eax		// dummy id
	call	SetRep_Old
	add	esp,12
	add	esi,12
	sub	edi,12
	jnz	nxt
  done:
	pop	edi
	pop	esi
	jmp	CharSel_Old
	align	16
  }
}




NAKED
void UpdAff_Hook()
{
  __asm {
	mov	eax,ds:[0x66da7c]
	inc	dword ptr [eax+0x14]	// update the player's Chat affiliation
	jmp	UpdAff_Old
	align	16
  }
}


void FASTCALL store_ship( UINT ship )
{
  ships.insert( ship );
}


NAKED
void SetAff_Hook()
{
  __asm {
	push	eax
	mov	ecx,[eax+0xb0]
	mov	target,ecx
	call	store_ship
	pop	eax
	mov	edx,[eax+0x104]
	ret
  }
}


bool FASTCALL test_ship( UINT ship )
{
  ShipSet::iterator iter = ships.find( ship );
  if (iter == ships.end())
    return false;
  ships.erase( iter );
  return true;
}


NAKED
void SetName_Hook()
{
  __asm {
	push	ecx
	mov	ecx,edx
	call	test_ship
	test	al,al
	pop	ecx
	jz	done
	mov	eax,[esi+0x900]
	mov	[esp+0x54],eax
	ret	8
  done:
	jmp	SetName_Old
  }
}


NAKED
void Target_Hook()
{
  __asm {
	mov	eax,[ebp+0x844]
	cmp	target,0
	je	done
	cmp	target,eax
	jne	reset
	xor	eax,eax
  reset:
	mov	target,0
  done:
	ret
  }
}


NAKED
void Land_Hook()
{
  __asm {
	mov	eax,[esp+8]
	mov	ecx,0x673378
	mov	eax,[eax]
	sub	eax,[ecx]
	jne	done
	mov	[ecx],eax
  done:
	jmp	Land_Old
	align	16
  }
}


void Patch()
{
  ProtectX( ADDR_LIST,	    4 );
  ProtectX( ADDR_ARROW,     2 );
  ProtectX( ADDR_DIR,	    5 );
  ProtectX( ADDR_SETNAME,   6 );
  ProtectX( ADDR_TARGET,    4 );
  ProtectX( ADDR_SETAFF,    4 );
  ProtectW( ADDR_CHARSEL,   4 );
//ProtectW( ADDR_LAND,	    4 );
//ProtectW( ADDR_SETREP,    4 );
//ProtectW( ADDR_NEWPLAYER, 4 );
//ProtectW( ADDR_UPDAFF,    4 );

  NEWABS( ADDR_NEWPLAYER, NewPlayer );
  NEWABS( ADDR_SETREP, SetRep );
  NEWABS( ADDR_CHARSEL, CharSel );
  NEWABS( ADDR_UPDAFF, UpdAff );
  NEWOFS( ADDR_LIST, List );
  CALL(   ADDR_DIR, Dir );
  *ADDR_ARROW = 0x01A8; 	// test al,1
  NEWABS( ADDR_SORT, Sort );
  CALL1(  ADDR_SETAFF, SetAff );
  NEWOFS( ADDR_SETNAME, SetName );
  CALL1(  ADDR_TARGET, Target );
  NEWABS( ADDR_LAND, Land );
}


BOOL WINAPI DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved )
{
  if (fdwReason == DLL_PROCESS_ATTACH)
    Patch();

  return TRUE;
}
