/*
  ChatRepSrv.cpp - Transmit existing player affiliations to a new player.

  Jason Hood, 11 June, 2014.

  When a new player logs on, the affiliation is sent to existing players, but
  existing players' affiliations are not sent to the new player - this plugin
  addresses that.

  v1.01, 17 & 18 June, 2014:
  + send change of affiliation to all players.
*/

#include "Server.h"
#include "patch.h"


#define ADDR_CHARSEL (0x6b5ff89+1 - 0x6b30000)
#define ADDR_AFF     (0x6d0c550+2 - 0x6ce0000)
#define ADDR_CLIENT  (0x6d8fb14   - 0x6ce0000)


PBYTE Client;
DWORD CharSel_Old;
DWORD Aff_Old, Aff_New, Aff_Ret;

struct Rep
{
  UINT	player;
  int	rep;
  UINT	fac;
};

struct Sizes
{
  size_t size;
  size_t max;
};


Rep* FASTCALL StoreRep( Rep* store, Sizes* size )
{
  PlayerData* pd = 0;

  while ((pd = Players.traverse_active( pd )))
  {
    if (size->size + sizeof(Rep) > size->max)
      break;
    size->size += sizeof(Rep);
    UINT fac;
    Reputation::Vibe::GetAffiliation( pd->rep, fac );
    store->player = pd->id;
    store->rep	  = pd->rep;
    store->fac	  = fac;
    ++store;
  }

  return store;
}


NAKED
void CharSel_Hook()
{
  __asm {
	mov	ecx, [ebx]
	mov	edx, esi
	call	StoreRep
	mov	[ebx], eax
	jmp	CharSel_Old
	align	16
  }
}


void SetAff( const int& rep, const UINT& fac )
{
  struct RepData
  {
    int   rep;
    float value;
    UINT  faction;
    int   dunno1;
    PVOID dunno2;	// array of two 4-byte values (Reputation::Relation)
    BYTE  dunno3;	// count of items in above
    char  fmtstr[2][24];
    WCHAR str[24];	// guess, max player name
  } rd;

  memset( &rd, 0, sizeof(rd) );
  rd.rep = rep;
  rd.faction = fac;

  PlayerData* pd = 0;
  while ((pd = Players.traverse_active( pd )))
  {
    if (pd->rep != rd.rep)
    {
      Reputation::Vibe::GetGroupFeelingsTowards( pd->rep, fac, rd.value );
      __asm {
	lea	eax,rd
	push	eax
	mov	eax,pd
	push	[eax+0x348]
	mov	ecx,Client
	mov	ecx,[ecx]
	mov	edx,[ecx]
	call	dword ptr [edx+0x108]	// SETREPUTATION
      }
    }
  }
}


NAKED
void Aff_Hook()
{
  __asm {
	push	[esp+12]
	push	[esp+8+4]
	push	[esp+4+8]
	call	Aff_Old
	add	esp,12
	jmp	SetAff
	align	16
  }
}


void Patch( void )
{
  PBYTE dll = (PBYTE)GetModuleHandle( "remoteclient.dll" );
  ProtectX( dll+ADDR_CHARSEL, 4 );
  NEWOFS( dll+ADDR_CHARSEL, CharSel );

  dll = (PBYTE)GetModuleHandle( "server.dll" );
  ProtectX( dll+ADDR_AFF, 4 );
  INDIRECT( dll+ADDR_AFF, Aff );
  Client = (PBYTE)(dll+ADDR_CLIENT);
}


BOOL WINAPI DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved )
{
  if (fdwReason == DLL_PROCESS_ATTACH)
    Patch();

  return TRUE;
}
